"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DecreaseDynamoDBProvisionedRcuWcuTrick = void 0;
const tslib_1 = require("tslib");
const aws_sdk_1 = tslib_1.__importDefault(require("aws-sdk"));
const chalk_1 = tslib_1.__importDefault(require("chalk"));
class DecreaseDynamoDBProvisionedRcuWcuTrick {
    constructor() {
        this.ddbClient = new aws_sdk_1.default.DynamoDB();
    }
    getMachineName() {
        return DecreaseDynamoDBProvisionedRcuWcuTrick.machineName;
    }
    async getCurrentState(task, currentState, options) {
        const tableNames = await this.listTableNames(task);
        const subListr = task.newListr([], {
            concurrent: 10,
            exitOnError: false,
            rendererOptions: {
                collapse: true,
            },
        });
        if (!tableNames || tableNames.length === 0) {
            task.skip(chalk_1.default.dim('no DynamoDB tables found'));
            return subListr;
        }
        subListr.add(tableNames.map((tableName) => {
            const tableState = {
                name: tableName,
            };
            currentState.push(tableState);
            return {
                title: tableName,
                task: async (ctx, task) => this.getTableState(task, tableState),
                options: {
                    persistentOutput: true,
                },
            };
        }));
        return subListr;
    }
    async conserve(task, currentState, options) {
        const subListr = task.newListr([], {
            concurrent: 10,
            exitOnError: false,
            rendererOptions: { collapse: true },
        });
        if (currentState && currentState.length > 0) {
            for (const table of currentState) {
                subListr.add({
                    title: `${chalk_1.default.greenBright(table.name)}`,
                    task: (ctx, task) => this.conserveTableProvisionedRcuWcu(task, table, options),
                    options: {
                        persistentOutput: true,
                    },
                });
            }
        }
        else {
            task.skip(chalk_1.default.dim(`no DynamoDB tables found`));
        }
        return subListr;
    }
    async restore(task, currentState, options) {
        const subListr = task.newListr([], {
            concurrent: 10,
            exitOnError: false,
            rendererOptions: { collapse: true },
        });
        if (currentState && currentState.length > 0) {
            for (const table of currentState) {
                subListr.add({
                    title: `${chalk_1.default.greenBright(table.name)}`,
                    task: (ctx, task) => this.restoreTableProvisionedRcuWcu(task, table, options),
                    options: {
                        persistentOutput: true,
                    },
                });
            }
        }
        else {
            task.skip(chalk_1.default.dim(`no DynamoDB tables was conserved`));
        }
        return subListr;
    }
    async listTableNames(task) {
        const tableNames = [];
        // TODO Add logic to go through all pages
        task.output = 'fetching page 1...';
        tableNames.push(...((await this.ddbClient.listTables({ Limit: 100 }).promise())
            .TableNames || []));
        return tableNames;
    }
    async getTableState(task, tableState) {
        task.output = 'fetching table information...';
        const provisionedThroughput = (await this.ddbClient
            .describeTable({ TableName: tableState.name })
            .promise()).Table.ProvisionedThroughput;
        if (!provisionedThroughput) {
            tableState.provisionedThroughput = false;
            return;
        }
        tableState.provisionedThroughput = true;
        tableState.rcu = provisionedThroughput.ReadCapacityUnits;
        tableState.wcu = provisionedThroughput.WriteCapacityUnits;
        task.output = 'done';
    }
    async conserveTableProvisionedRcuWcu(task, tableState, options) {
        if (!tableState.provisionedThroughput) {
            task.skip(chalk_1.default.dim(`provisioned throughput is not configured`));
            return;
        }
        if (tableState.wcu < 2 && tableState.rcu < 2) {
            task.skip(chalk_1.default.dim(`provisioned RCU/WCU is already at minimum of 1`));
            return;
        }
        if (options.dryRun) {
            task.skip(chalk_1.default.dim('skipped, would configure RCU = 1 WCU = 1'));
            return;
        }
        task.output = `configuring RCU = 1 WCU = 1 ...`;
        await this.ddbClient
            .updateTable({
            TableName: tableState.name,
            ProvisionedThroughput: {
                ReadCapacityUnits: 1,
                WriteCapacityUnits: 1,
            },
        })
            .promise();
        task.output = `configured RCU = 1 WCU = 1`;
    }
    async restoreTableProvisionedRcuWcu(task, tableState, options) {
        if (!tableState.provisionedThroughput) {
            task.skip(chalk_1.default.dim(`provisioned throughput was not configured`));
            return;
        }
        if (options.dryRun) {
            task.skip(chalk_1.default.dim(`skipped, would configure RCU = ${tableState.rcu} WCU = ${tableState.wcu}`));
            return;
        }
        if (!tableState.rcu || !tableState.wcu) {
            task.skip(chalk_1.default.dim(`skipped, RCU = ${tableState.rcu} WCU = ${tableState.wcu} are not configured correctly`));
            return;
        }
        task.output = `configuring ${tableState.rcu} WCU = ${tableState.wcu} ...`;
        await this.ddbClient
            .updateTable({
            TableName: tableState.name,
            ProvisionedThroughput: {
                ReadCapacityUnits: tableState.rcu,
                WriteCapacityUnits: tableState.wcu,
            },
        })
            .promise();
        task.output = `configured RCU = ${tableState.rcu} WCU = ${tableState.wcu}`;
    }
}
exports.DecreaseDynamoDBProvisionedRcuWcuTrick = DecreaseDynamoDBProvisionedRcuWcuTrick;
DecreaseDynamoDBProvisionedRcuWcuTrick.machineName = 'decrease-dynamodb-provisioned-rcu-wcu';
//# sourceMappingURL=decrease-dynamodb-provisioned-rcu-wcu.trick.js.map